@@ -19,42 +19,83 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from xdot import *
+try:
+    # intentionally use from xdot, instead of from .xdot, because we want to use local xdot for Python2 and system xdot for Python3
+    from xdot.ui.elements import *
+    from xdot.ui.animation import *
+    from xdot.dot.lexer import *
+    from xdot.dot.parser import *
+    import subprocess
+except:
+    from xdot import *
 
+from distutils.version import LooseVersion
+
+# Python 3 renamed the unicode type to str, the old str type has been replaced by bytes.
+if sys.version_info[0] >= 3:
+    unicode = str
 
 __all__ = ['WxDotWindow', 'WxDotFrame']
 
 # We need to get the wx version with built-in cairo support
-import wxversion
-if wxversion.checkInstalled("2.8"):
-    wxversion.select("2.8")
-else:
-    print("wxversion 2.8 is not installed, installed versions are {}".format(wxversion.getInstalled()))
+try:
+    import wxversion
+    if wxversion.checkInstalled("2.8"):
+        wxversion.select("2.8")
+    else:
+        print("wxversion 2.8 is not installed, installed versions are {}".format(wxversion.getInstalled()))
+except:
+    pass  # Python3
+
 import wx
 import wx.lib.wxcairo as wxcairo
 
 # This is a crazy hack to get this to work on 64-bit systems
-if 'wxMac' in wx.PlatformInfo:
-  pass # Implement if necessary
-elif 'wxMSW' in wx.PlatformInfo:
-  pass # Implement if necessary
-elif 'wxGTK' in wx.PlatformInfo:
-  import ctypes
-  gdkLib = wx.lib.wxcairo._findGDKLib()
-  gdkLib.gdk_cairo_create.restype = ctypes.c_void_p
+try:
+    if 'wxMac' in wx.PlatformInfo:
+        pass # Implement if necessary
+    elif 'wxMSW' in wx.PlatformInfo:
+        pass # Implement if necessary
+    elif 'wxGTK' in wx.PlatformInfo:
+        import ctypes
+        gdkLib = wx.lib.wxcairo._findGDKLib()
+        gdkLib.gdk_cairo_create.restype = ctypes.c_void_p
+except:
+    pass  # Python3
+
+class MyXDotParser(XDotParser):
+
+  def __init__(self, xdotcode):
+    XDotParser.__init__(self,xdotcode)
+    self.subgraph_shapes = {}
+
+  def parse_subgraph(self):
+    shapes_before = set(self.shapes)
+
+    id = XDotParser.parse_subgraph(self)
+
+    new_shapes = set(self.shapes) - shapes_before
+    self.subgraph_shapes[id.decode()] = [s for s in new_shapes if not any([s in ss for ss in self.subgraph_shapes.values()])]
+
+    return id
+
+  def parse(self):
+    graph = XDotParser.parse(self)
+    graph.subgraph_shapes = self.subgraph_shapes
+    return graph
 
 class WxDragAction(object):
   def __init__(self, dot_widget):
     self.dot_widget = dot_widget
 
   def on_button_press(self, event):
-    x,y = event.GetPositionTuple()
+    x,y = event.GetPosition()
     self.startmousex = self.prevmousex = x
     self.startmousey = self.prevmousey = y
     self.start()
 
   def on_motion_notify(self, event):
-    x,y = event.GetPositionTuple()
+    x,y = event.GetPosition()
     deltax = self.prevmousex - x
     deltay = self.prevmousey - y
     self.drag(deltax, deltay)
@@ -62,7 +103,7 @@ class WxDragAction(object):
     self.prevmousey = y
 
   def on_button_release(self, event):
-    x,y = event.GetPositionTuple()
+    x,y = event.GetPosition()
     self.stopmousex = x
     self.stopmousey = y
     self.stop()
@@ -271,7 +312,10 @@ class WxDotWindow(wx.Panel):
 
   ### Cursor manipulation
   def set_cursor(self, cursor_type):
-    self.cursor = wx.StockCursor(cursor_type)
+    if LooseVersion(wx.__version__) >= LooseVersion('4.0'):
+        self.cursor = wx.Cursor(cursor_type)
+    else:
+        self.cursor = wx.StockCursor(cursor_type)
     self.SetCursor(self.cursor)
 
   ### Zooming methods
@@ -363,7 +407,7 @@ class WxDotWindow(wx.Panel):
     return WxNullAction(self)
 
   def OnMouse(self, event):
-    x,y = event.GetPositionTuple()
+    x,y = event.GetPosition()
 
     item = None
 
@@ -436,8 +480,9 @@ class WxDotWindow(wx.Panel):
     self.filter = filter
 
   def set_dotcode(self, dotcode, filename='<stdin>'):
-    if isinstance(dotcode, unicode):
-      dotcode = dotcode.encode('utf8')
+    # Python 3 renamed the unicode type to str, the old str type has been replaced by bytes.
+    if sys.version_info[0] < 3 and isinstance(dotcode, unicode):
+          dotcode = dotcode.encode('utf8')
     p = subprocess.Popen(
       [self.filter, '-Txdot'],
       stdin=subprocess.PIPE,
@@ -448,7 +493,7 @@ class WxDotWindow(wx.Panel):
     )
     xdotcode, error = p.communicate(dotcode)
     if p.returncode != 0:
-      print "ERROR PARSING DOT CODE", error
+      print("ERROR PARSING DOT CODE {}".format(error))
       dialog = Gtk.MessageDialog(type=gtk.MESSAGE_ERROR,
                      message_format=error,
                      buttons=gtk.BUTTONS_OK)
@@ -468,8 +513,8 @@ class WxDotWindow(wx.Panel):
       # Store references to subgraph states
       self.subgraph_shapes = self.graph.subgraph_shapes
 
-    except ParseError, ex:
-      print "ERROR PARSING XDOT CODE"
+    except ParseError as ex:
+      print("ERROR PARSING XDOT CODE")
       dialog = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,
                      message_format=str(ex),
                      buttons=gtk.BUTTONS_OK)
@@ -484,7 +529,10 @@ class WxDotWindow(wx.Panel):
   def set_xdotcode(self, xdotcode):
     """Set xdot code."""
     #print xdotcode
-    parser = XDotParser(xdotcode)
+    if sys.version_info[0] >= 3:
+      parser = MyXDotParser(bytes(str(xdotcode).encode("utf-8")))
+    else:
+      parser = XDotParser(xdotcode)
     self.graph = parser.parse()
     self.highlight = None
     #self.zoom_image(self.zoom_ratio, center=True)
@@ -547,11 +595,11 @@ Refresh: R",
       try:
         self.open_file(path)
 
-      except IOError, error:
+      except IOError as error:
         dlg = wx.MessageDialog(self, 'Error opening file\n' + str(error))
         dlg.ShowModal()
 
-      except UnicodeDecodeError, error:
+      except UnicodeDecodeError as error:
         dlg = wx.MessageDialog(self, 'Error opening file\n' + str(error))
         dlg.ShowModal()
 
@@ -575,7 +623,7 @@ Refresh: R",
       fp = file(filename, 'rt')
       self.set_dotcode(fp.read(), filename)
       fp.close()
-    except IOError, ex:
+    except IOError as ex:
       """
       dlg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,
                   message_format=str(ex),
